import { Component, h, Event, State } from '@stencil/core';
import Draggabilly from 'draggabilly';
import { debounce } from 'lodash-es';
// import { getWindowInnerWidth, getWindowInnerHeight } from '../../services/windowSize';
export class QuickomDraggable {
  constructor() {
    this.draggable = null;
    this.element = null;
    this.isVisible = true;
    this.visibleObserver = null;
    this.dragPosition = {
      left: 0,
      right: 0,
      top: 0,
      bottom: 0,
      height: 0,
      width: 0,
    };
    this.onResize = null;
  }
  async onRefDraggableEl(el) {
    this.element = el;
  }
  componentDidLoad() {
    this.initDraggable();
    // this.observeVisibility();
    this.listenWindowResize();
  }
  listenWindowResize() {
    let onResize = debounce(() => {
      this.calibratePosition();
      // if (!this.isVisible) {
      //   this.calibratePosition();
      // }
    }, 500);
    this.onResize = onResize;
    window.addEventListener('resize', onResize);
  }
  observeVisibility() {
    var _a;
    (_a = this.visibleObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
    this.visibleObserver = new IntersectionObserver((changes, _observer) => {
      changes.forEach(change => {
        this.isVisible = change.intersectionRatio > 0;
      });
    }, {
      root: null,
      rootMargin: '0px',
      threshold: [0],
    });
    this.visibleObserver.observe(this.element);
  }
  initDraggable() {
    var _a;
    (_a = this.draggable) === null || _a === void 0 ? void 0 : _a.destroy();
    let computedStyle = getComputedStyle(this.element);
    let right = +computedStyle.getPropertyValue('--widget-position-right').replace('px', '');
    let bottom = +computedStyle.getPropertyValue('--widget-position-bottom').replace('px', '');
    let top = +computedStyle.getPropertyValue('--widget-position-top').replace('px', '');
    let left = +computedStyle.getPropertyValue('--widget-position-left').replace('px', '');
    this.draggable = new Draggabilly(this.element, {
      containment: window.document.body,
    });
    this.draggable.measureContainment();
    if (bottom) {
      top = this.draggable.containSize.height - bottom;
      this.dragPosition.bottom = bottom;
    }
    if (right) {
      left = this.draggable.containSize.width - right;
      this.dragPosition.right = right;
    }
    this.dragPosition.top = top || 0;
    this.dragPosition.left = left || 0;
    if (top || left) {
      this.draggable.setPosition(left || 0, top || 0);
    }
    this.saveCurrentSize();
    this.draggable
      .on('staticClick', () => {
      this.staticClick.emit(true);
    })
      .on('dragEnd', () => {
      this.saveCurrentSize();
    });
  }
  getCurrentSize() {
    this.draggable.measureContainment();
    let top = this.draggable.position.y;
    let left = this.draggable.position.x;
    let right = this.draggable.containSize.width;
    let bottom = this.draggable.containSize.height;
    let height = bottom + this.draggable.relativeStartPosition.y;
    let width = right + this.draggable.relativeStartPosition.x;
    return { top, left, right, bottom, width, height };
  }
  saveCurrentSize() {
    let { top, left, right, bottom, height, width } = this.getCurrentSize();
    this.dragPosition.left = left;
    this.dragPosition.right = right;
    this.dragPosition.top = top;
    this.dragPosition.bottom = bottom;
    this.dragPosition.width = width;
    this.dragPosition.height = height;
  }
  calibratePosition() {
    let { width, height, left, top, right, bottom } = this.getCurrentSize();
    let { width: oldWidth, height: oldHeight } = this.dragPosition;
    let x = right < left ? left - (oldWidth - width) : left;
    let y = bottom < top ? top - (oldHeight - height) : top;
    this.draggable.setPosition(x, y);
    this.saveCurrentSize();
  }
  render() {
    return (h("div", { ref: el => this.onRefDraggableEl(el), class: "inline-block absolute" },
      h("slot", null)));
  }
  disconnectedCallback() {
    var _a, _b;
    (_a = this.draggable) === null || _a === void 0 ? void 0 : _a.destroy();
    (_b = this.visibleObserver) === null || _b === void 0 ? void 0 : _b.disconnect();
    window.removeEventListener('resize', this.onResize);
  }
  static get is() { return "quickom-draggable"; }
  static get originalStyleUrls() { return {
    "$": ["quickom-draggable.css"]
  }; }
  static get styleUrls() { return {
    "$": ["quickom-draggable.css"]
  }; }
  static get states() { return {
    "isVisible": {}
  }; }
  static get events() { return [{
      "method": "move",
      "name": "move",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "moveStart",
      "name": "moveStart",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "staticClick",
      "name": "staticClick",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
}
