import EventEmitter3 from 'eventemitter3';
export class ElementEventEmitter extends EventEmitter3 {
  constructor(element) {
    super();
    this.element = element;
  }
  asObservable() {
    let listeners = [];
    let elListeners = {};
    let addEventListener = (type) => {
      if (!elListeners[type]) {
        let listener = (...params) => {
          this.emit(type, ...params);
        };
        elListeners[type] = listener;
        this.element.addEventListener(type, listener);
      }
    };
    let removeAllEventListener = () => {
      Object.entries(elListeners).forEach(e => {
        this.element.removeEventListener(e[0], e[1]);
      });
      elListeners = {};
    };
    let observable = {
      on: (type, cb) => {
        listeners.push({ type, cb });
        this.on(type, cb);
        addEventListener(type);
        return observable;
      },
      once: (type, cb) => {
        listeners.push({ type, cb });
        this.once(type, cb);
        addEventListener(type);
        return observable;
      },
      removeListener: (type, cb) => {
        this.removeListener(type, cb);
        return observable;
      },
      off: (type, cb) => {
        this.off(type, cb);
        return observable;
      },
      removeAllListener() {
        listeners.forEach(({ type, cb }) => {
          this.off(type, cb);
        });
        listeners.length = 0;
        removeAllEventListener();
        return observable;
      },
      unsubscribe() {
        return observable.removeAllListener();
      },
      reEmit(eventEmitter, eventNames) {
        for (let e of eventNames) {
          observable.on(e, (...args) => {
            eventEmitter.emit(e, ...args);
          });
        }
        return observable;
      }
    };
    return observable;
  }
  reEmit(e, eventNames) {
    let observable = e.asObservable();
    for (let e of eventNames) {
      observable.on(e, (...args) => {
        this.emit(e, ...args);
      });
    }
    return observable;
  }
}
