import { r as registerInstance, c as createEvent, h } from './index-58e46480.js';
import './_commonjsHelpers-9943807e.js';
import { I as IframeMessenger, a as IframeSession } from './iframeSession-f3b01436.js';

const quickomChatHookCss = "quickom-chat-hook{}";

const QuickomChatHook = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.chatMessage = createEvent(this, "chatMessage", 7);
    this.domain = 'https://widget.quickom.com';
    this.memberId = '';
    this.apiKey = '';
    this.iframeElement = null;
    this.iframeMessenger = null;
    this.iframeSession = null;
  }
  getUrl() {
    return `${this.domain}/chat-hook`;
  }
  onIframeRef(el) {
    this.iframeElement = el;
  }
  // private reloadIframe() {
  //   if (this.iframeElement) {
  //     this.iframeElement.src = this.getUrl();
  //   }
  // }
  onLoadIframe() {
    this.iframeMessenger = new IframeMessenger(this.iframeElement);
    this.iframeMessenger.start();
    this.iframeSession = IframeSession.init(this.iframeMessenger);
    this.hookChat();
    this.iframeSession.on('message', this.onChatBoxMessage.bind(this));
  }
  hookChat() {
    if (this.iframeSession && this.memberId && this.apiKey) {
      this.iframeSession.send({
        resource: 'chat',
        method: 'hookChat',
        params: [{ memberId: this.memberId, apiKey: this.apiKey }],
      });
    }
  }
  onChatBoxMessage(data) {
    switch (data.method) {
      case 'receiveMessage':
        this.chatMessage.emit(data.params[0]);
        break;
    }
  }
  render() {
    return h("iframe", { onLoad: () => this.onLoadIframe(), ref: el => this.onIframeRef(el), src: this.getUrl(), width: "0", height: "0", frameborder: "0" });
  }
  onChangeMemberId() {
    this.hookChat();
  }
  onChangeApiKey() {
    this.hookChat();
  }
  disconnectedCallback() {
    this.iframeSession.destroy();
    this.iframeMessenger.stop();
    this.iframeMessenger = null;
    this.iframeSession = null;
  }
  static get watchers() { return {
    "memberId": ["onChangeMemberId"],
    "apiKey": ["onChangeApiKey"]
  }; }
};
QuickomChatHook.style = quickomChatHookCss;

export { QuickomChatHook as quickom_chat_hook };
